import sys
sys.path.insert(0, '..')

import pytest

import os
import uocapp_flask
import tempfile
import json
import uocapp_coap_cli
import mota_dummy

@pytest.fixture
def client(request):
    db_fd, uocapp_flask.app.config['DATABASE'] = tempfile.mkstemp()
    client = uocapp_flask.app.test_client()
    with uocapp_flask.app.app_context():
        uocapp_flask.init_db('./db/uocapp_test.sql')
    
    forgimg = open('./mapa.png', 'rb')
    fimgd, uocapp_flask.app.config["IMG_MAPA"] = tempfile.mkstemp()
    fimg = os.fdopen(fimgd, 'wb')
    fimg.write(forgimg.read())
    fimg.close()
    forgimg.close()
    
    cli = uocapp_coap_cli.init(50000)
    cli.ackTimeout    = 1
    cli.respTimeout   = 1
    srv = mota_dummy.init()

    def teardown():
        os.close(db_fd)
        os.unlink(uocapp_flask.app.config['DATABASE'])
        cli.close()
        srv.close()
    request.addfinalizer(teardown)

    return client

def test_sin_login(client):
    """ Test sin login """
    _test_sensores(client)
    _test_parametros(client)
    _test_motas(client)
    _test_ultimos_valores_motas(client)
    _test_valores_mota_sensor(client)

def _test_sensores(client):
    rv = client.get('/sensores')
    c = _checks(rv)
    d = json.loads(c)
    assert len(d) == 4
    assert d[0]['id'] == 3
    assert d[0]['desc'] == 'Aceleracion (ADXL346)'
    assert d[0]['uni'] == 'm/s'
    assert d[0]['conv'] == '2 * VAL'
    assert d[1]['id'] == 2
    assert d[1]['desc'] == 'Humedad (SHT21)'
    assert d[1]['uni'] == '%HR'
    assert d[1]['conv'] == 'VAL + 1'
    assert d[2]['id'] == 4
    assert d[2]['desc'] == 'Luz (MAX44009)'
    assert d[2]['uni'] == 'Lux'
    assert d[2]['conv'] == 'VAL / 2'
    assert d[3]['id'] == 1
    assert d[3]['desc'] == 'Temperatura (SHT21)'
    assert d[3]['uni'] == 'C'
    assert d[3]['conv'] == 'VAL'

def _test_parametros(client):
    rv = client.get('/parametros')
    c = _checks(rv)
    d = json.loads(c)
    assert len(d) == 6
    assert d[0]['id'] == 4
    assert d[0]['len'] == 4
    assert d[0]['tipo'] == 1
    assert d[0]['mota'] == 0
    assert d[0]['desc'] == 'Coordenada X'
    assert d[0]['desctipo'] == 'Numerico'
    assert d[1]['id'] == 5
    assert d[1]['len'] == 4
    assert d[1]['tipo'] == 1
    assert d[1]['mota'] == 0
    assert d[1]['desc'] == 'Coordenada Y'
    assert d[1]['desctipo'] == 'Numerico'
    assert d[2]['id'] == 6
    assert d[2]['len'] == 255
    assert d[2]['tipo'] == 2
    assert d[2]['mota'] == 0
    assert d[2]['desc'] == 'Descripcion'
    assert d[2]['desctipo'] == 'Texto'
    assert d[3]['id'] == 3
    assert d[3]['len'] == 255
    assert d[3]['tipo'] == 2
    assert d[3]['mota'] == 0
    assert d[3]['desc'] == 'Informacion libre'
    assert d[3]['desctipo'] == 'Texto'
    assert d[4]['id'] == 1
    assert d[4]['len'] == 255
    assert d[4]['tipo'] == 3
    assert d[4]['mota'] == 1
    assert d[4]['desc'] == 'Servidores'
    assert d[4]['desctipo'] == 'Lista IPv6 separadas por ,'
    assert d[5]['id'] == 2
    assert d[5]['len'] == 4
    assert d[5]['tipo'] == 1
    assert d[5]['mota'] == 1
    assert d[5]['desc'] == 'Tiempo entre muestras (segundos)'
    assert d[5]['desctipo'] == 'Numerico'

def _test_motas(client):
    rv = client.get('/motas')
    c = _checks(rv)
    d = json.loads(c)
    assert len(d) == 1
    assert d[0]['id'] == 1
    assert d[0]['desc'] == "Mota 1"
    assert d[0]['ip'] == "[::1]:5684"
    assert d[0]['X'] == '6'
    assert d[0]['Y'] == '7'
    assert d[0]['fecha'] != None

def _test_ultimos_valores_motas(client):
    rv = client.get('/motas/1/valores')
    c = _checks(rv)
    d = json.loads(c)
    assert len(d) == 4
    assert d[0]['id'] == 3
    assert d[0]['valor'] == 33
    assert d[0]['fecha'] != None
    assert d[0]['desc'] == 'Aceleracion (ADXL346)'
    assert d[0]['uni'] == 'm/s'
    assert d[1]['id'] == 2
    assert d[1]['valor'] == 22
    assert d[1]['fecha'] != None
    assert d[1]['desc'] == 'Humedad (SHT21)'
    assert d[1]['uni'] == '%HR'
    assert d[2]['id'] == 4
    assert d[2]['valor'] == 44
    assert d[2]['fecha'] != None
    assert d[2]['desc'] == 'Luz (MAX44009)'
    assert d[2]['uni'] == 'Lux'
    assert d[3]['id'] == 1
    assert d[3]['valor'] == 11
    assert d[3]['fecha'] != None
    assert d[3]['desc'] == 'Temperatura (SHT21)'
    assert d[3]['uni'] == 'C'
    
def _test_valores_mota_sensor(client):
    rv = client.get('/motas/1/valores/1/10')
    c = _checks(rv)
    d = json.loads(c)
    assert len(d) == 2
    assert d[0]['valor'] == 1
    assert d[0]['fecha'] != None
    assert d[0]['uni'] == 'C'
    assert d[1]['valor'] == 11
    assert d[1]['fecha'] != None
    assert d[1]['uni'] == 'C'

def test_login(client):
    """ Test login """
    _test_login(client)
    _test_get_login(client)
    _test_logout(client)
    _test_get_login_vacio(client)
    _test_login_fallo(client)

def _test_login(client):
    din = {"usuario":"administrador", "password":"password"}
    rv = client.post('/login',data=json.dumps(din), content_type = 'application/json')
    assert rv.status_code == 200

def _test_get_login(client):
    rv = client.get('/login')
    c = _checks(rv)
    d = json.loads(c)
    assert len(d) == 1
    assert d["usuario"] == "administrador"

def _test_logout(client):
    rv = client.delete('/login')
    assert rv.status_code == 200    

def _test_get_login_vacio(client):
    rv = client.get('/login')
    c = _checks(rv)
    d = json.loads(c)
    assert len(d) == 1
    assert d["usuario"] == None
    
def _test_login_fallo(client):
    din = {"usuario":"administrador", "password":"password1"}
    rv = client.post('/login',data=json.dumps(din), content_type = 'application/json')
    assert rv.status_code == 401

def test_adm(client):
    """ Test de administracion de la red (consultas) """
    _test_adm_motas_nologin(client)
    _test_login(client)
    _test_adm_motas(client)
    _test_logout(client)
    
    _test_adm_motas_configuracion_nologin(client)
    _test_login(client)
    _test_adm_motas_configuracion(client)
    _test_logout(client)
    
    _test_adm_mota_status_nologin(client)
    _test_login(client)
    _test_adm_mota_status(client)
    _test_logout(client)

def _test_adm_motas_nologin(client):
    rv = client.get('/adm/motas')
    assert rv.status_code == 401

def _test_adm_motas(client):
    rv = client.get('/adm/motas')
    c = _checks(rv)
    d = json.loads(c)
    assert len(d) == 1
    assert d[0]['id'] == 1
    assert d[0]['desc'] == 'Mota 1'
    assert d[0]['ip'] == '[::1]:5684'
    assert d[0]['fecha'] != None
    assert d[0]['fechaInfo'] != None
    assert d[0]['sinc'] == 111111
    assert d[0]['sincMota'] == 111111
    assert d[0]['alerta'] == None
    assert d[0]['X'] == '6'
    assert d[0]['Y'] == '7'

def _test_adm_motas_configuracion_nologin(client):
    rv = client.get('/adm/motas/1/configuracion')
    assert rv.status_code == 401

def _test_adm_motas_configuracion(client):
    rv = client.get('/adm/motas/1/configuracion')
    c = _checks(rv)
    d = json.loads(c)
    assert len(d) == 6
    assert d[0]['id'] == 1
    assert d[0]['valor'] == '[::1]:1,[::1]:2'
    assert d[0]['fecha'] != None
    assert d[1]['id'] == 2
    assert d[1]['valor'] == '300'
    assert d[1]['fecha'] != None
    assert d[2]['id'] == 3
    assert d[2]['valor'] == 'Libre'
    assert d[2]['fecha'] != None
    assert d[3]['id'] == 4
    assert d[3]['valor'] == '6'
    assert d[3]['fecha'] != None
    assert d[4]['id'] == 5
    assert d[4]['valor'] == '7'
    assert d[4]['fecha'] != None
    assert d[5]['id'] == 6
    assert d[5]['valor'] == 'Mota 1'
    assert d[5]['fecha'] != None

def _test_adm_mota_status_nologin(client):
    rv = client.get('/adm/motas/1/status')
    assert rv.status_code == 401

def _test_adm_mota_status(client):
    rv = client.get('/adm/motas/1/status')
    c = _checks(rv)
    d = json.loads(c)
    assert len(d) == 1
    assert d[0]['id'] == 1
    assert d[0]['desc'] == 'Estado bateria'
    assert d[0]['comparacion'] == '<='
    assert d[0]['valor_critico'] == 5
    assert d[0]['valor'] == 4
    assert d[0]['fecha'] != None
    assert d[0]['uni'] == 'V'

def test_adm2(client):
    """ Test de administracion de la red (actualizacion 1 motas) """
    _test_adm_mota_configuracion_modificacion_nologin(client)
    _test_login(client)
    _test_adm_mota_configuracion_modificacion(client)
    _test_adm_motas_configuracion_tras_modificacion(client)
    _test_logout(client)

def _test_adm_mota_configuracion_modificacion_nologin(client):
    din = [{"id":2, "valor":"600"}, {"id":6, "valor":"Mota 2.1"}, {"id":4, "valor":"11"}, {"id":5, "valor":"22"}]
    rv = client.put('/adm/motas/1/configuracion',data=json.dumps(din), content_type = 'application/json')
    assert rv.status_code == 401
    
def _test_adm_mota_configuracion_modificacion(client):
    din = [{"id":2, "valor":"600"}, {"id":6, "valor":"Mota 2.1"}, {"id":4, "valor":"11"}, {"id":5, "valor":"22"}]
    rv = client.put('/adm/motas/1/configuracion',data=json.dumps(din), content_type = 'application/json')
    assert rv.status_code == 200

def _test_adm_motas_configuracion_tras_modificacion(client):
    rv = client.get('/adm/motas/1/configuracion')
    c = _checks(rv)
    d = json.loads(c)
    assert len(d) == 6
    assert d[0]['id'] == 1
    assert d[0]['valor'] == '[::1]:1,[::1]:2'
    assert d[0]['fecha'] != None
    assert d[1]['id'] == 2
    assert d[1]['valor'] == '600'
    assert d[1]['fecha'] != None
    assert d[2]['id'] == 3
    assert d[2]['valor'] == 'Libre'
    assert d[2]['fecha'] != None
    assert d[3]['id'] == 4
    assert d[3]['valor'] == '11'
    assert d[3]['fecha'] != None
    assert d[4]['id'] == 5
    assert d[4]['valor'] == '22'
    assert d[4]['fecha'] != None
    assert d[5]['id'] == 6
    assert d[5]['valor'] == 'Mota 2.1'
    assert d[5]['fecha'] != None

    rv = client.get('/adm/motas')
    c = _checks(rv)
    d = json.loads(c)
    assert len(d) == 1
    assert d[0]['id'] == 1
    assert d[0]['desc'] == 'Mota 2.1'
    assert d[0]['ip'] == '[::1]:5684'
    assert d[0]['fecha'] != None
    assert d[0]['fechaInfo'] != None
    assert d[0]['sinc'] != 111111
    assert d[0]['sincMota'] != None
    assert d[0]['alerta'] == None
    assert d[0]['X'] == '11'
    assert d[0]['Y'] == '22'

def test_adm3(client):
    """ Test de administracion de la red (actualizaciones) """
    _test_adm_motas_configuracion_modificacion_nologin(client)
    _test_login(client)
    _test_adm_motas_configuracion_modificacion(client)
    _test_adm_motas_configuracion_tras_modificacion2(client)
    _test_logout(client)

def _test_adm_motas_configuracion_modificacion_nologin(client):
    din = [{"id":2, "valor":"600"}, {"id":6, "valor":"Mota 2.1"}, {"id":4, "valor":"11"}, {"id":5, "valor":"22"}]
    rv = client.put('/adm/motas/configuracion',data=json.dumps(din), content_type = 'application/json')
    assert rv.status_code == 401
    
def _test_adm_motas_configuracion_modificacion(client):
    din = [{"id":1, "valor":"[::1]:1,[::1]:2,[::1]:3"}, {"id":2, "valor":"600"}, {"id":6, "valor":"Mota 2.1"}, {"id":4, "valor":"11"}, {"id":5, "valor":"22"}]
    rv = client.put('/adm/motas/configuracion',data=json.dumps(din), content_type = 'application/json')
    assert rv.status_code == 200

def _test_adm_motas_configuracion_tras_modificacion2(client):
    rv = client.get('/adm/motas/1/configuracion')
    c = _checks(rv)
    d = json.loads(c)
    assert len(d) == 6
    assert d[0]['id'] == 1
    assert d[0]['valor'] == '[::1]:1,[::1]:2,[::1]:3'
    assert d[0]['fecha'] != None
    assert d[1]['id'] == 2
    assert d[1]['valor'] == '600'
    assert d[1]['fecha'] != None
    assert d[2]['id'] == 3
    assert d[2]['valor'] == 'Libre'
    assert d[2]['fecha'] != None
    assert d[3]['id'] == 4
    assert d[3]['valor'] == '6'
    assert d[3]['fecha'] != None
    assert d[4]['id'] == 5
    assert d[4]['valor'] == '7'
    assert d[4]['fecha'] != None
    assert d[5]['id'] == 6
    assert d[5]['valor'] == 'Mota 1'
    assert d[5]['fecha'] != None
	
    rv = client.get('/adm/motas')
    c = _checks(rv)
    d = json.loads(c)
    assert len(d) == 1
    assert d[0]['id'] == 1
    assert d[0]['desc'] == 'Mota 1'
    assert d[0]['ip'] == '[::1]:5684'
    assert d[0]['fecha'] != None
    assert d[0]['fechaInfo'] != None
    assert d[0]['sinc'] != 111111
    assert d[0]['sincMota'] != None
    assert d[0]['alerta'] == None
    assert d[0]['X'] == '6'
    assert d[0]['Y'] == '7'

def test_adm4(client):
    """ Test de administracion de la red (actualizaciones) """
    _test_adm_borra_mota_modificacion_nologin(client)
    _test_login(client)
    _test_adm_borra_mota_modificacion(client)
    _test_adm_motas_trasborrado(client)
    _test_logout(client)
    
def _test_adm_borra_mota_modificacion_nologin(client):
    rv = client.delete('/adm/motas/1')
    assert rv.status_code == 401

def _test_adm_borra_mota_modificacion(client):
    rv = client.delete('/adm/motas/1')
    assert rv.status_code == 200

def _test_adm_motas_trasborrado(client):
    rv = client.get('/adm/motas')
    c = _checks(rv)
    d = json.loads(c)
    assert len(d) == 0

def test_administracion(client):
    """ Test de administracion de la aplicacion (consultas) """ 
    _test_get_administracion_nologin(client)
    _test_login(client)
    _test_get_administracion(client)
    _test_logout(client)  

def _test_get_administracion_nologin(client):
    rv = client.get('/administracion/sensores')
    assert rv.status_code == 401

def _test_get_administracion(client):
    rv = client.get('/administracion/sensores')
    c = _checks(rv)
    d = json.loads(c)
    assert len(d) == 4
    assert d[0]['id'] == 1
    assert d[0]['descripcion'] == 'Temperatura (SHT21)'
    assert d[0]['unidades'] == 'C'
    assert d[0]['conversion'] == 'VAL'
    assert d[1]['id'] == 2
    assert d[1]['descripcion'] == 'Humedad (SHT21)'
    assert d[1]['unidades'] == '%HR'
    assert d[1]['conversion'] == 'VAL + 1'
    assert d[2]['id'] == 3
    assert d[2]['descripcion'] == 'Aceleracion (ADXL346)'
    assert d[2]['unidades'] == 'm/s'
    assert d[2]['conversion'] == '2 * VAL'
    assert d[3]['id'] == 4
    assert d[3]['descripcion'] == 'Luz (MAX44009)'
    assert d[3]['unidades'] == 'Lux'
    assert d[3]['conversion'] == 'VAL / 2'

def test_administracion2(client):
    """ Test de administracion de la aplicacion (actualiazaciones) """    
    _test_update_administracion_nologin(client)
    _test_login(client)
    _test_update_administracion(client)
    _test_get_administracion_tras_modificacion(client)
    _test_logout(client)
    
    _test_delete_administracion_nologin(client)
    _test_login(client)
    _test_delete_administracion(client)
    _test_get_administracion_tras_borrado(client)
    _test_logout(client)

def _test_update_administracion_nologin(client):
    din = {"id":1, "descripcion":"Temperatura (SHT21) modificado", "unidades":"F", "conversion":"VAL + VAL"}
    rv = client.post('/administracion/sensores',data=json.dumps(din), content_type = 'application/json')
    assert rv.status_code == 401

def _test_update_administracion(client):
    din = {"id":1, "descripcion":"Temperatura (SHT21) modificado", "unidades":"F", "conversion":"VAL + VAL"}
    rv = client.post('/administracion/sensores',data=json.dumps(din), content_type = 'application/json')
    assert rv.status_code == 200

def _test_get_administracion_tras_modificacion(client):
    rv = client.get('/administracion/sensores')
    c = _checks(rv)
    d = json.loads(c)
    assert len(d) == 4
    assert d[0]['id'] == 1
    assert d[0]['descripcion'] == 'Temperatura (SHT21) modificado'
    assert d[0]['unidades'] == 'F'
    assert d[0]['conversion'] == 'VAL + VAL'
    assert d[1]['id'] == 2
    assert d[1]['descripcion'] == 'Humedad (SHT21)'
    assert d[1]['unidades'] == '%HR'
    assert d[1]['conversion'] == 'VAL + 1'
    assert d[2]['id'] == 3
    assert d[2]['descripcion'] == 'Aceleracion (ADXL346)'
    assert d[2]['unidades'] == 'm/s'
    assert d[2]['conversion'] == '2 * VAL'
    assert d[3]['id'] == 4
    assert d[3]['descripcion'] == 'Luz (MAX44009)'
    assert d[3]['unidades'] == 'Lux'
    assert d[3]['conversion'] == 'VAL / 2'

def _test_delete_administracion_nologin(client):
    rv = client.delete('/administracion/sensores/1')
    assert rv.status_code == 401

def _test_delete_administracion(client):
    rv = client.delete('/administracion/sensores/1')
    assert rv.status_code == 200

def _test_get_administracion_tras_borrado(client):
    rv = client.get('/administracion/sensores')
    c = _checks(rv)
    d = json.loads(c)
    assert len(d) == 3
    assert d[0]['id'] == 2
    assert d[0]['descripcion'] == 'Humedad (SHT21)'
    assert d[0]['unidades'] == '%HR'
    assert d[0]['conversion'] == 'VAL + 1'
    assert d[1]['id'] == 3
    assert d[1]['descripcion'] == 'Aceleracion (ADXL346)'
    assert d[1]['unidades'] == 'm/s'
    assert d[1]['conversion'] == '2 * VAL'
    assert d[2]['id'] == 4
    assert d[2]['descripcion'] == 'Luz (MAX44009)'
    assert d[2]['unidades'] == 'Lux'
    assert d[2]['conversion'] == 'VAL / 2'

def test_mapa(client):
    """ Test de administracion del mapa """
    _test_get_mapa_info(client)
    _test_get_mapa(client)
    
    _test_update_mapa_nologin(client)
    _test_login(client)
    _test_update_mapa(client)
    _test_get_mapa_info_tras_modificacion(client)
    _test_get_mapa_tras_modificacion(client)
    _test_logout(client)

def _test_get_mapa_info(client):
    rv = client.get('/mapa/info')
    c = _checks(rv)
    d = json.loads(c)
    assert len(d) == 1
    assert d[0]['X'] == 1024
    assert d[0]['Y'] == 768

def _test_get_mapa(client):
    rv = client.get('/mapa')
    assert rv.content_type == 'image/png'
    assert rv.status_code == 200
    assert len(rv.get_data()) == 752819
    
def _test_update_mapa_nologin(client):
    din = {"X":1, "Y":1, "mapa":"MTIzNDU2"}
    rv = client.post('/mapa',data=json.dumps(din), content_type = 'application/json')
    assert rv.status_code == 401

def _test_update_mapa(client):
    din = {"X":1, "Y":1, "mapa":"MTIzNDU2"}
    rv = client.post('/mapa',data=json.dumps(din), content_type = 'application/json')
    assert rv.status_code == 200

def _test_get_mapa_info_tras_modificacion(client):
    rv = client.get('/mapa/info')
    c = _checks(rv)
    d = json.loads(c)
    assert len(d) == 1
    assert d[0]['X'] == 1
    assert d[0]['Y'] == 1
    
def _test_get_mapa_tras_modificacion(client):
    rv = client.get('/mapa')
    assert rv.content_type == 'image/png'
    assert rv.status_code == 200
    assert len(rv.get_data()) == 6

def _checks(rv):
    assert rv.content_type == 'application/json'
    assert rv.status_code == 200
    return rv.get_data()
