import sys
sys.path.insert(0, '..')

import pytest
from coapUocapp import coap, coapDefines as d, coapOption
import time
import uocapp_coap_srv
import uocapp_flask
import tempfile
import os
import json

@pytest.fixture
def client(request):
    db_fd, uocapp_flask.app.config['DATABASE'] = tempfile.mkstemp()
    client = uocapp_flask.app.test_client()
    with uocapp_flask.app.app_context():
        uocapp_flask.init_db('./db/uocapp_test.sql')
    srv = uocapp_coap_srv.init(50002)
    
    def teardown():
        os.close(db_fd)
        os.unlink(uocapp_flask.app.config['DATABASE'])
        srv.close()
    request.addfinalizer(teardown)
    
    return client

@pytest.fixture
def coapCli(request):
    coapCli = coap.coap(udpPort=50001)
    
    def teardown():
        coapCli.close()
    request.addfinalizer(teardown)
    
    return coapCli

def test_srv(client, coapCli):
    s = '{"s":{"1":1, "2":2}, "e":{"1":30}, "f":112233}'
    coapCli.POST(uri='coap://[::1]:50002/uocapp', options=[coapOption.ContentFormat([d.FORMAT_JSON])], payload=[ord(b) for b in s])

    din = {"usuario":"administrador", "password":"password"}
    rv = client.post('/login',data=json.dumps(din), content_type = 'application/json')
    assert rv.status_code == 200
    
    rv = client.get('/adm/motas')
    assert rv.status_code == 200
    dat = json.loads(rv.get_data())
    assert len(dat) == 2
    assert dat[0]["id"] == 1
    assert dat[1]["id"] == 2
    assert dat[1]["ip"] == '[::1]:50001'
    assert dat[1]["desc"] == '[::1]:50001'
    assert dat[1]["fechaInfo"] != None
    assert dat[1]["sinc"] == None
    assert dat[1]["sincMota"] ==112233
    assert dat[1]["X"] == '-1'
    assert dat[1]["Y"] == '-1'
    assert dat[1]["alerta"] == None
    
    rv = client.get('/adm/motas/2/configuracion')
    assert rv.status_code == 200
    dat = json.loads(rv.get_data())
    assert len(dat) == 6
    assert dat[0]['id'] == 1
    assert dat[0]['valor'] == '[::1]:5555'
    assert dat[0]['fecha'] != None
    assert dat[1]['id'] == 2
    assert dat[1]['valor'] == '500'
    assert dat[1]['fecha'] != None
    assert dat[2]['id'] == 3
    assert dat[2]['valor'] == ''
    assert dat[2]['fecha'] != None
    assert dat[3]['id'] == 4
    assert dat[3]['valor'] == '-1'
    assert dat[3]['fecha'] != None
    assert dat[4]['id'] == 5
    assert dat[4]['valor'] == '-1'
    assert dat[4]['fecha'] != None
    assert dat[5]['id'] == 6
    assert dat[5]['valor'] == '[::1]:50001'
    assert dat[5]['fecha'] != None
    
    rv = client.get('/motas/2/valores')
    assert rv.status_code == 200
    dat = json.loads(rv.get_data())
    assert len(dat) == 2
    assert dat[0]['id'] == 2
    assert dat[0]['valor'] == 3
    assert dat[0]['fecha'] != None
    assert dat[0]['desc'] == 'Humedad (SHT21)'
    assert dat[0]['uni'] == '%HR'
    assert dat[1]['id'] == 1
    assert dat[1]['valor'] == 1
    assert dat[1]['fecha'] != None
    assert dat[1]['desc'] == 'Temperatura (SHT21)'
    assert dat[1]['uni'] == 'C'
    

    rv = client.get('/motas/2/valores/1/15')
    assert rv.status_code == 200
    dat = json.loads(rv.get_data())
    assert len(dat) == 1
    assert dat[0]['valor'] == 1
    assert dat[0]['fecha'] != None
    assert dat[0]['uni'] != None
    
    rv = client.get('/adm/motas/2/status')
    assert rv.status_code == 200
    dat = json.loads(rv.get_data())
    assert len(dat) == 1
    assert dat[0]['id'] == 1
    assert dat[0]['valor'] == 60
    assert dat[0]['fecha'] != None

    time.sleep(1)

    s = '{"s":{"1":11, "2":22}, "e":{"1":2}, "f":11223344}'
    coapCli.POST(uri='coap://[::1]:50002/uocapp', options=[coapOption.ContentFormat([d.FORMAT_JSON])], payload=[ord(b) for b in s])

    rv = client.get('/adm/motas')
    assert rv.status_code == 200
    dat = json.loads(rv.get_data())
    assert len(dat) == 2
    assert dat[0]["id"] == 1
    assert dat[1]["id"] == 2
    assert dat[1]["ip"] == '[::1]:50001'
    assert dat[1]["desc"] == '[::1]:50001'
    assert dat[1]["fechaInfo"] != None
    assert dat[1]["sinc"] == None
    assert dat[1]["sincMota"] ==11223344
    assert dat[1]["X"] == '-1'
    assert dat[1]["Y"] == '-1'
    assert dat[1]["alerta"] == 'Bateria baja'
    
    rv = client.get('/motas/2/valores')
    assert rv.status_code == 200
    dat = json.loads(rv.get_data())
    assert len(dat) == 2
    assert dat[0]['id'] == 2
    assert dat[0]['valor'] == 23
    assert dat[0]['fecha'] != None
    assert dat[0]['desc'] == 'Humedad (SHT21)'
    assert dat[1]['id'] == 1
    assert dat[1]['valor'] == 11
    assert dat[1]['fecha'] != None
    assert dat[1]['desc'] == 'Temperatura (SHT21)'

    rv = client.get('/motas/2/valores/1/10')
    assert rv.status_code == 200
    dat = json.loads(rv.get_data())
    assert len(dat) == 2
    assert dat[0]['valor'] == 1
    assert dat[0]['fecha'] != None
    assert dat[1]['valor'] == 11
    assert dat[1]['fecha'] != None
    
    rv = client.get('/adm/motas/2/status')
    assert rv.status_code == 200
    dat = json.loads(rv.get_data())
    assert len(dat) == 1
    assert dat[0]['id'] == 1
    assert dat[0]['valor'] == 4
    assert dat[0]['fecha'] != None
