"""
Modulo principal de la aplicacion uocapp
Genera las rutas y arranca la aplicacion Flask
"""

# imports
import sqlite3
from flask import Flask, Response, g, json, request, session, send_file
from contextlib import closing
import uocapp_dao as dao
from functools import wraps
import types
import threading
import uocapp_coap_cli
from uocapp_logging import log_request, log_request_error
import random
import base64

# aplicacion
app = Flask(__name__)
# cargar configuracion por defecto de las variables previas
app.config.from_pyfile('./cfg/uocapp.cfg')

###################################################
# Funciones relacionadas con la BBDD
###################################################
def connect_db():
    """
    Genera una conexion a la BBDD SQLite3
    """
    return sqlite3.connect(app.config['DATABASE'])

def init_db(sql):
    """
    Inicializa la BBDD cargando el esquema y los datos basicos
    definidos en el fichero db/uocapp.sql
    """
    with closing(connect_db()) as db:
        with app.open_resource(sql, mode='r') as f:
            db.cursor().executescript(f.read())
            db.commit()

###################################################
# Funciones relacionadas con las peticiones
###################################################
@app.before_request
def _before_request():
    """
    Tareas que se ejecutan antes de iniciar una peticion
    Crea la conexion a bbdd y la guarda en g.db
    """
    g.db = connect_db()

@app.teardown_request
def _teardown_request(exception):
    """
    Tareas que se ejecutan al finalizar una peticion
    Cierra la conexion a bbdd guardada en g.db
    """
    db = getattr(g, 'db', None)
    if db is not None:
        db.close()

###################################################
# Decorators
###################################################
def _login_required(f):
    """
    Decorator usado para exigir el login antes de algunas peticiones
    El login se conoce porque el usuario esta guardado en g.usuario
    """
    @wraps(f)
    def decorated_function(*args, **kwargs):
        usuario = session.has_key('USUARIO') and session['USUARIO'] != None and len(session['USUARIO']) > 0
        if not usuario:
            return Response(status=401)
        return f(*args, **kwargs)
    return decorated_function

###################################################
# Funciones auxiliares
###################################################
def _resp_consulta(bbdd, labels):
    resultado = map(lambda(l): {labels[idx]: l[idx] for idx in range(len(l))}, bbdd)
    return Response(response=json.dumps(resultado), status=200, mimetype='application/json')

def _motas(labels, alerta):
    datos = dao.motasExtendido(g.db)
    resultado = []
    resAux = {}
    oldid = None
    for dato in datos:
        if oldid == None or oldid != dato[0]:
            oldid = dato[0]
            resAux = {}
            for idx in range(len(labels)):
                resAux[labels[idx]] = dato[idx]
            resultado.append(resAux)
            if alerta:
                resAux["alerta"] = dao.motaAlerta(g.db, dato[0])
        resAux[str(dato[6])] = dato[7]
    return resultado


###################################################
# URLs de atencion a peticiones sin login
###################################################
@app.route('/sensores', methods=['GET'])
def sensores():
    """
    GET /sensores Listado de los sensores
    Salida: [{"id":1, "desc":'...', "uni":'...', "conv":'...'}, ]
    """
    try:
        log_request(request, session)
        return _resp_consulta(dao.sensores(g.db), ["id","desc","uni","conv"])
    except Exception as e:
        log_request_error(request, e)

@app.route('/parametros', methods=['GET'])
def parametros():
    """
    GET /parametros Listado de los parametros
    Salida: [{"id":1, "desc":'...', "tipo":1, "mota":1, "len":1, "desctipo":'...'}, ]
    """
    try:
        log_request(request, session)
        return _resp_consulta(dao.parametros(g.db), ["id","desc","tipo","mota","len","desctipo"])
    except Exception as e:
        log_request_error(request, e)

@app.route('/motas', methods=['GET'])
def motas():
    """
    GET /motas Listado de las motas con la informacion basica
    Salida: [{"id":1, "ip":'...', "fecha":yyyy-MM-dd hh:mm:ss'}, ]
    """
    try:
        log_request(request, session)
        resultado = _motas(["id","ip","fecha"], False)
        return Response(response=json.dumps(resultado), status=200, mimetype='application/json')

    except Exception as e:
        log_request_error(request, e)

@app.route('/motas/<int:id>/valores', methods=['GET'])
def ultimos_valores_motas(id):
    """
    GET /motas/:id/valores listado de los ultimos valores de sensores de una mota
    Salida: [{"id":1, "desc":'...', "fecha": 'yyyy-MM-dd hh:mm:ss', "valor": 1, "uni": '...'}, ]
    """
    try:
        log_request(request, session)
        return _resp_consulta(dao.ultimos_valores_motas(g.db, id), ["id","desc","fecha","valor","uni"])
    except Exception as e:
        log_request_error(request, e)

@app.route('/motas/<int:id>/valores/<int:idSensor>/<int:limite>', methods=['GET'])
def valores_mota_sensor(id, idSensor, limite):
    """
    GET /motas/:id/valores/:idSensor Listado de los ultimos N valores de un sensor en una mota
    Salida: [{ "fecha": 'yyyy-MM-dd hh:mm:ss', "valor": 1, "uni":'...'}, ]
    """
    try:
        log_request(request, session)
        return _resp_consulta(dao.valores_mota_sensor(g.db, id, idSensor, limite), ["fecha","valor","uni"])
    except Exception as e:
        log_request_error(request, e)

###################################################
# URLs de atencion a peticiones con login
###################################################
@app.route('/adm/motas', methods=['GET'])
@_login_required
def adm_motas():
    """
    GET /adm/motas Listado de motas para el administrador
    Salida: [{"id":1, "ip":'...', "fecha":'yyyy-MM-dd hh:mm:ss', "fechaInfo":'yyyy-MM-dd hh:mm:ss', "alerta":'...'}, ]
    """
    try:
        log_request(request, session)
        resultado = _motas(["id","ip","fecha","fechaInfo","sinc","sincMota"], True)
        return Response(response=json.dumps(resultado), status=200, mimetype='application/json')
    except Exception as e:
        log_request_error(request, e)

@app.route('/adm/motas/<int:id>', methods=['DELETE'])
@_login_required
def adm_borra_mota(id):
    """
    DELETE /adm/motas/:id Borra una mota del sistema
    """
    try:
        log_request(request, session)
        dao.borra_mota(g.db, id)
        return Response(status=200)
    except Exception as e:
        log_request_error(request, e)

@app.route('/adm/motas/<int:id>/configuracion', methods=['GET'])
@_login_required
def adm_mota_configuracion(id):
    """
    GET /adm/motas/:id/configuracion Listado de los valores de los parametros
        de configuracion de una mota para el administrador
    Salida:  [{"id": 1, "desc":'...', "valor":'...'}, ]
    """
    try:
        log_request(request, session)
        return _resp_consulta(dao.valores_parametros(g.db, id), ["id","valor","fecha"])
    except Exception as e:
        log_request_error(request, e)

@app.route('/adm/motas/<int:id>/status', methods=['GET'])
@_login_required
def adm_mota_status(id):
    """
    GET /adm/motas/:id/status Listado de los valores de los parametros de
        status de una mota para el administrador
    Salida: [{"id": 1, "desc":'...', "valor":'...', "valor_critico":'...', "fecha": 'yyyy-MM-dd hh:mm:ss'}, ]
    """
    try:
        log_request(request, session)
        return _resp_consulta(dao.mota_status(g.db, id), ["id","desc","comparacion","valor","valor_critico","fecha","uni"])
    except Exception as e:
        log_request_error(request, e)

@app.route('/adm/motas/<int:id>/configuracion', methods=['PUT'])
@_login_required
def adm_mota_configuracion_modificacion(id):
    """
    PUT /adm/motas/:id/configuracion Modifica el valor de parametros de
        configuracion en una mota
    Entrada: [{"id": 1, "valor":'...'}, ]
    """
    try:
        log_request(request, session)
        paramsMota = _parametros_mota()

        #Se construye el mensaje CoAP
        parametros = request.get_json(silent=True)
        if type(parametros) != types.ListType or len(parametros) == 0:
            return Response(status=400)
        coapMsj = {}
        for param in parametros:
            if param["id"] == None or param["valor"] == None:
                return Response(status=400)
            if param["id"] in paramsMota:
                if param["valor"].isdigit():
                    coapMsj[param["id"]] = int(param["valor"])
                else:
                    coapMsj[param["id"]] = param["valor"]
        coapMsj[0] = random.randint(1,99999)

        _adm_mota_configuracion_modificacion_aux(id, parametros, coapMsj)
    
        return Response(status=200)
    except Exception as e:
        log_request_error(request, e)

def _adm_mota_configuracion_modificacion_aux(id, parametros, coapMsj):
    #Se recupera la ip de la mota
    bbdd = dao.motaId(g.db, id)
    if len(bbdd) == 0:
        return Response(status=400)
    ip = bbdd[0][1];

    #Se invoca al cliente CoAP solo si hay mas parametros que el 0
    # solo en ese caso se envia la sincronizacion a la base de datos
    sinc = None
    if len(coapMsj) > 1:
        resp = uocapp_coap_cli.post(ip, coapMsj)
        sinc = coapMsj[0]

    #Se actualizan los parametros que se conservan en la BBDD
    dao.inserta_valores_parametros(g.db, id, parametros, sinc)

def _parametros_mota():
    ids_parametros = []
    parametros = dao.parametros(g.db)
    for parametro in parametros:
        if parametro[3] == 1:
            ids_parametros.append(parametro[0])
    return ids_parametros

@app.route('/adm/motas/configuracion', methods=['PUT'])
@_login_required
def adm_motas_configuracion_modificacion():
    """
    PUT /adm/motas/configuracion Modifica el valor de parametros de
        configuraci0n en una mota
    Entrada: [{"id": 1, "valor":'...'}, ]
    """
    try:
        log_request(request, session)
        paramsMota = _parametros_mota()

        #Se construye el mensaje CoAP
        parametros = request.get_json(silent=True)
        if type(parametros) != types.ListType or len(parametros) == 0:
            return Response(status=400)
        coapMsj = {}
        motaparams = []
        for param in parametros:
            if param["id"] == None or param["valor"] == None:
                return Response(status=400)
            if param["id"] in paramsMota:
                if param["valor"].isdigit():
                    coapMsj[param["id"]] = int(param["valor"])
                else:
                    coapMsj[param["id"]] = param["valor"]
                motaparams.append(param)
                
        coapMsj[0] = random.randint(1,99999)
            
        #Se invoca al cliente CoAP
        motas = dao.motasSimple(g.db)
        for mota in motas:
            _adm_mota_configuracion_modificacion_aux(mota[0], motaparams, coapMsj)

        return Response(status=200)
    except Exception as e:
        log_request_error(request, e)

###################################################
# Funciones de login
###################################################
@app.route('/login', methods=['POST'])
def login():
    """
    POST /login Login en la aplicacion
    Entrada: {"usuario":'...', "password":'...'}
    """
    try:
        parametros = request.get_json(silent=True)
        if type(parametros) != types.DictType or parametros["usuario"] == None or parametros["password"] == None:
            return Response(status=400)
        
        if dao.login(g.db, parametros["usuario"], parametros["password"]):
            session['USUARIO'] = parametros["usuario"]
            return Response(status=200)

        return Response(status=401)
    except Exception as e:
        log_request_error(request, e)

@app.route('/login', methods=['DELETE'])
@_login_required
def logout():
    """
    DELETE /login Logout en la aplicacion
    """
    try:
        log_request(request, session)
        session['USUARIO'] = None
        return Response(status=200)
    except Exception as e:
        log_request_error(request, e)

@app.route('/login', methods=['GET'])
def get_login():
    """
    GET /login Recupera el usuario logado en la aplicacion
    """
    try:
        log_request(request, session)
        if session.has_key('USUARIO'):
            resultado = {"usuario": session['USUARIO']}
        else:
            resultado = {"usuario": ""}
        return Response(response=json.dumps(resultado), status=200, mimetype='application/json')
    except Exception as e:
        log_request_error(request, e)

###################################################
# Funciones de administracion de tablas maestras
###################################################
@app.route('/administracion/<tabla>', methods=['GET'])
@_login_required
def get_administracion(tabla):
    """
    GET /administracion/:tabla Consulta los datos de una tabla maestra
        para administracion
    """
    try:
        log_request(request, session)

        metadatos = dao.desc_tabla(g.db, tabla)
        labels = []
        for metadato in metadatos:
            labels.append(metadato[1])
        return _resp_consulta(dao.datos_tabla(g.db, tabla), labels)
    except Exception as e:
        log_request_error(request, e)

@app.route('/administracion/<tabla>', methods=['POST'])
@_login_required
def update_administracion(tabla):
    """
    POST /administracion/:tabla Actualiza los datos de una tabla maestra
        para administracion (menos la tabla de usuarios)
    Entrada: {campo:valor,... de los campos de la tabla}
    """
    try:
        log_request(request, session)
        
        parametros = request.get_json(silent=True)
        if tabla == 'usuarios':
            if type(parametros) != types.DictType or parametros["usuario"] == None or parametros["password"] == None:
                return Response(status=400)
            dao.update_usuario(g.db, parametros)
        else:   
            if type(parametros) != types.DictType or parametros["id"] == None:
                return Response(status=400)
            dao.update_datos_tabla(g.db, tabla, parametros)
        return Response(status=200)
    except Exception as e:
        log_request_error(request, e)

@app.route('/administracion/<tabla>/<id>', methods=['DELETE'])
@_login_required
def delete_administracion(tabla, id):
    """
    DELETE /administracion/:tabla/:id Elimina los datos de una tabla maestra
        para administracion
    """
    try:
        log_request(request, session)

        dao.delete_datos_tabla(g.db, tabla, id)
        return Response(status=200)
    except Exception as e:
        log_request_error(request, e)

###################################################
# Funciones relacionadas con el mapa
###################################################
@app.route('/mapa', methods=['GET'])
def get_mapa():
    """
    GET /mapa Obtiene la imagen del mapa
    """
    try:
        log_request(request, session)
        return send_file(app.config["IMG_MAPA"], mimetype='image/png')
    except Exception as e:
        log_request_error(request, e)

@app.route('/mapa/info', methods=['GET'])
def get_mapa_info():
    """
    GET /mapa/info Obtiene las dimensones del mapa
    Salida: [{ "fecha": 'yyyy-MM-dd hh:mm:ss', "valor": 1}, ]
    """
    try:
        log_request(request, session)
        return _resp_consulta(dao.get_mapa_info(g.db), ["X","Y"])
    except Exception as e:
        log_request_error(request, e)

@app.route('/mapa', methods=['POST'])
@_login_required
def update_mapa():
    """
    POST /mapa Guarda un nuevo mapa
    """
    try:
        log_request(request, session)
        parametros = request.get_json(silent=True)
        
        with open(app.config["IMG_MAPA"], 'wb') as fout:
            fout.write(base64.b64decode(parametros['mapa']))
        mapa = dao.update_mapa(g.db, parametros['X'], parametros['Y'])

        return Response(status=200)
    except Exception as e:
        log_request_error(request, e)
