/**
\brief Aplicacion UOCAPP
*/

#include "opendefs.h"
#include "uocapp.h"
#include "opencoap.h"
#include "openqueue.h"
#include "packetfunctions.h"
#include "openserial.h"
#include "idmanager.h"
#include "scheduler.h"

//=========================== defines =========================================
/**
\brief Path del recurso CoAP
*/
const uint8_t uocapp_coap_path0[] = "uocapp";
/**
\brief Path del recurso CoAP de la aplicacion web
*/
const uint8_t recursoCoapAppWeb[] = "uocapp";

const uint8_t digitos[] = {'0','1','2','3','4','5','6','7','8','9'};
const uint8_t digitos_hex[] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};


//=========================== variables =======================================
/**
\brief Variable que utiliza la aplicacion para uso interno
*/
uocapp_vars_t uocapp_vars;


//=========================== prototypes ======================================
/**
\brief Funcion que se invoca cuando se recibe un mensaje CoAP para el recurso
*/
owerror_t uocapp_coap_receive(OpenQueueEntry_t*, coap_header_iht*, coap_option_iht* );
/**
\brief Funcion que se invoca despues de enviar un mensaje CoAP
*/
void uocapp_coap_sendDone(OpenQueueEntry_t*, owerror_t);
/**
\brief Funcion que se invoca a traves del planificador
*/
void uocapp_tarea_planificada(opentimer_id_t);
/**
\brief Funcion que recibe la peticion POST CoAP y modifica los parámetros de actuacion
*/
void uocapp_modificar_parametros(uint8_t*);
/**
\brief Funcion que parsea la peticion JSON
*/
uint32_t uocapp_parsea_peticion(uint8_t*, uint32_t, uint8_t*);
/**
\brief Parsea la lista de sensores activos (ids separados por ,)
*/
void uocapp_parsea_lista_sensores(uint8_t*, _uocapp_trata_sensor);
/**
\brief Parsea la lista de servidores (lista de direcciones [ipv6]:puerto)
*/
void uocapp_parsea_param_servidor(uint8_t*, _uocapp_trata_direccion_servidor);
/**
\brief Activa el servidor dado
*/
void uocapp_trata_sensor(uint8_t);
/**
\brief Asigna la ip:puerto a cada servidor de la lista de configuracion
*/
void uocapp_trata_direccion_servidor(uint8_t*, uint16_t, uint8_t);
/**
\brief Funcion que se invoca despues de enviar un mensaje CoAP
*/
void uocapp_coap_send(uint8_t*,uint32_t);
/**
\brief Funcion que se invoca para modificar la planificación de la tarea
*/
void uocapp_modifica_planificacion(uint32_t);
/**
\brief Funcion que se invoca para leer los sensores de estado
*/
uint32_t uocapp_lectura_sensores(uint8_t*, uint8_t);
/**
\brief Funcion que se invoca para leer los sensores de estado
*/
uint32_t uocapp_lectura_sensores_estado(uint8_t*);
/**
\brief Funcion que se ejecuta desde el planificador como respuesta al timer
*/
void uocapp_tarea_planificada_exec(void);

/**
\brief Funciones auxiliares
*/
uint32_t uocapp_len(uint8_t*);
uint32_t uocapp_cc_s(uint8_t*, uint32_t, char*, uint8_t);
uint32_t uocapp_cc_sd(uint8_t*, uint32_t, uint8_t);
uint32_t uocapp_cc_d(uint8_t*, uint32_t, uint16_t);
uint32_t uocapp_cc_ld(uint8_t*, uint32_t, uint32_t);
uint32_t uocapp_cc_pv(uint8_t*, uint32_t, uint8_t, uint32_t);
uint32_t uocapp_cc_hex(uint8_t*, uint32_t, uint8_t);
uint32_t uocapp_atoi(uint8_t*, uint8_t);

//=========================== public ==========================================

/**
\brief Inicializa el modulo en openapps
*/
void uocapp_init() {
   uint8_t iCnt = 0;
   uint8_t jCnt = 0;
   uint8_t numSensors = 0;
   opensensors_resource_desc_t* opensensors_resource_temp;

   // No ejecutar en la mota GATEWAY
   if(idmanager_getIsDAGroot()==TRUE) return;

   // Inicializar opensensor
   opensensors_init();

   memset(&uocapp_vars,0,sizeof(uocapp_vars_t));

   // Registrar el modulo CoAP
   uocapp_vars.coap.path0len             = sizeof(uocapp_coap_path0)-1;
   uocapp_vars.coap.path0val             = (uint8_t*)(&uocapp_coap_path0);
   uocapp_vars.coap.path1len             = 0;
   uocapp_vars.coap.path1val             = NULL;
   uocapp_vars.coap.componentID          = COMPONENT_UOCAPP;
   uocapp_vars.coap.discoverable         = TRUE;
   uocapp_vars.coap.callbackRx           = &uocapp_coap_receive;
   uocapp_vars.coap.callbackSendDone     = &uocapp_coap_sendDone;
   opencoap_register(&uocapp_vars.coap);


   // Inicializando temporizador
   if (!uocapp_vars.tarea.enEjecucion) {
	 uocapp_vars.tarea.periodo = DEFAULT_TIME * 1000;
	 uocapp_vars.tarea.enEjecucion = TRUE;
	 uocapp_vars.tarea.timerId = opentimers_start(
		uocapp_vars.tarea.periodo,
	    TIMER_PERIODIC,
	    TIME_MS,
	    &uocapp_tarea_planificada);
   }

   numSensors = opensensors_getNumSensors();

   // Inicializando la lista de sensores
   for (iCnt = 0; iCnt < MAX_SENSORES; iCnt++) {
		uocapp_vars.sensores[iCnt].id = SENSORES_ACTIVOS[iCnt];
		uocapp_vars.sensores[iCnt].activo = TRUE;
		uocapp_vars.sensores[iCnt].sen = NULL;
		for (jCnt = 0; jCnt < numSensors; jCnt++) {
			opensensors_resource_temp = opensensors_getResource(jCnt);
			if (opensensors_resource_temp->sensorType == SENSORES_ACTIVOS[iCnt]) {
				uocapp_vars.sensores[iCnt].sen = opensensors_resource_temp;
			}
		}
   }

   // Inicializando la lista de sensores
   for (iCnt = 0; iCnt < MAX_ESTADO; iCnt++) {
   		uocapp_vars.estado[iCnt].id = SENSORES_ACTIVOS_ESTADO[iCnt];
   		uocapp_vars.estado[iCnt].idStatus = iCnt + 1;
   		uocapp_vars.estado[iCnt].sen = NULL;
   		for (jCnt = 0; jCnt < numSensors; jCnt++) {
   			opensensors_resource_temp = opensensors_getResource(jCnt);
   			if (opensensors_resource_temp->sensorType == SENSORES_ACTIVOS_ESTADO[iCnt]) {
   				uocapp_vars.estado[iCnt].sen = opensensors_resource_temp;
   			}
   		}
   }

   // Inicializando la lista de servidores destino
   uocapp_vars.servidores[0].activo = TRUE;
   uocapp_vars.servidores[0].puerto = WKP_UDP_COAP;
   memcpy(uocapp_vars.servidores[0].ipv6Addr, &ipAddr_openvisualizer, 16);
   for (iCnt = 1; iCnt < MAX_SERVIDORES; iCnt++) {
	   uocapp_vars.servidores[iCnt].activo = FALSE;
   }

   // Inicializando la sincronizacion
   uocapp_vars.sincronizacion = 0;

}

//=========================== private =========================================
/**
\brief Funcion que se invoca cuando se recibe un mensaje CoAP para el recurso
\param[in] msg mensje recibido / mensaje de respuesta
\param[in] coap_header cabecera CoAP
\param[in] coap_options opciones CoAP
\return Codigo de respuesta
*/
owerror_t uocapp_coap_receive(
      OpenQueueEntry_t* msg,
      coap_header_iht* coap_header,
      coap_option_iht* coap_options ) {

   owerror_t outcome;
   uint8_t buffer[127];
   uint8_t iCnt = 0;
   uint8_t jCnt = 0;
   uint8_t idx = 0;
   uint32_t len = 0;
   bool ok = FALSE;

   switch (coap_header->Code) {
      case COAP_CODE_REQ_GET:
		// La aplicacion no necesita este recurso, se aprovecha
		// para pruebas unitarias

        //=== reset packet payload (we will reuse this packetBuffer)
        msg->payload = &(msg->packet[127]);
        msg->length = 0;

		// prepara la respuesta con el valor de los parametros de configuracion
        buffer[0] = 0;

        // recuperamos el querystring
        for (iCnt=0;iCnt<MAX_COAP_OPTIONS;iCnt++) {
        	if (coap_options[iCnt].type == COAP_OPTION_NUM_URIQUERY) {

        		// si el querystring es 0 se envia la informacion de los parametros numericos
        		if (coap_options[iCnt].pValue[0] == '0') {
        			len = uocapp_cc_s(buffer, len, "{", 1);
        			len = uocapp_cc_pv(buffer, len, PARAM_SINCRONIZACION, uocapp_vars.sincronizacion);
        			len = uocapp_cc_s(buffer, len, ",", 1);
        			len = uocapp_cc_pv(buffer, len, PARAM_PERIODO, uocapp_vars.tarea.periodo);
        			len = uocapp_cc_s(buffer, len, ",\"", 2);
        			len = uocapp_cc_sd(buffer, len, PARAM_SENSORES_ACTIVOS);
        			len = uocapp_cc_s(buffer, len, "\":\"", 3);
        			for (iCnt = 0; iCnt < MAX_SENSORES; iCnt++) {
        				if (uocapp_vars.sensores[iCnt].activo) {
        					if (iCnt > 0) {
        						len = uocapp_cc_s(buffer, len, ",", 1);
        					}
        					len = uocapp_cc_sd(buffer, len, uocapp_vars.sensores[iCnt].id);
        				}
        			}
        			len = uocapp_cc_s(buffer, len, "\"}", 2);
        			ok = TRUE;
        		}

        		// si el querystring es 1x se envia el servidor de indice x en la lista de servidores
        		else if (coap_options[iCnt].pValue[0] == '1' && coap_options[iCnt].length == 2) {
        			idx = coap_options[iCnt].pValue[1] - '0';
        			if (idx < MAX_SERVIDORES && uocapp_vars.servidores[idx].activo) {
        				len = uocapp_cc_s(buffer, len, "{\"", 2);
        				len = uocapp_cc_sd(buffer, len, PARAM_SERVIDORES);
        				len = uocapp_cc_s(buffer, len, "\":\"[", 4);
        				for (jCnt = 0; jCnt < 16; jCnt++) {
        					if (jCnt > 0 && jCnt % 2 == 0) {
        						len = uocapp_cc_s(buffer, len, ":", 1);
        					}
        					len = uocapp_cc_hex(buffer, len, uocapp_vars.servidores[idx].ipv6Addr[jCnt]);
        				}
        				len = uocapp_cc_s(buffer, len, "]:", 2);
        				len = uocapp_cc_d(buffer, len, uocapp_vars.servidores[idx].puerto);
        				len = uocapp_cc_s(buffer, len, "\"}", 2);
        				ok = TRUE;
        			}
        		}

        		// si el querystring es s se envia la lectura de los sensores
        		else if (coap_options[iCnt].pValue[0] == 's' && coap_options[iCnt].length == 2) {
					idx = coap_options[iCnt].pValue[1] - '0';
        			len = uocapp_lectura_sensores(buffer, idx);
        			ok = TRUE;
        		}

        		// si el querystring es e se envia la lectura de los sensores de estado
        		else if (coap_options[iCnt].pValue[0] == 'e') {
        			len = uocapp_lectura_sensores_estado(buffer);
        			ok = TRUE;
        		}

        		// si el querystring es s se envia la lectura de los sensores
        		else if (coap_options[iCnt].pValue[0] == 'i') {
        			len = uocapp_cc_s(buffer, len, "[", 1);
        			len = uocapp_cc_sd(buffer, len, opensensors_getNumSensors());
        			for (jCnt = 0; jCnt < opensensors_getNumSensors(); jCnt++) {
        				len = uocapp_cc_s(buffer, len, ",", 1);
        				switch(opensensors_getResource(jCnt)->sensorType) {
        				case SENSOR_TEMPERATURE: len = uocapp_cc_s(buffer, len, "\"T\"", 3); break;
        				case SENSOR_HUMIDITY: len = uocapp_cc_s(buffer, len, "\"H\"", 3); break;
        				case SENSOR_LIGHT: len = uocapp_cc_s(buffer, len, "\"L\"", 3); break;
        				case SENSOR_XACCELERATION: len = uocapp_cc_s(buffer, len, "\"X\"", 3); break;
        				case SENSOR_YACCELERATION: len = uocapp_cc_s(buffer, len, "\"Y\"", 3); break;
        				case SENSOR_ZACCELERATION: len = uocapp_cc_s(buffer, len, "\"Z\"", 3); break;
        				case SENSOR_ADCTEMPERATURE: len = uocapp_cc_s(buffer, len, "\"t\"", 3); break;
        				case SENSOR_ADCVDD: len = uocapp_cc_s(buffer, len, "\"v\"", 3); break;
        				default: len = uocapp_cc_s(buffer, len, "\"?\"", 3); break;
        				};
        			}
        			len = uocapp_cc_s(buffer, len, "]", 1);
        			ok = TRUE;
        		}

        		// si el querystring es s se envia la lectura de los sensores
        		else if (coap_options[iCnt].pValue[0] == 'p' && coap_options[iCnt].length == 2) {
        			len = uocapp_cc_s(buffer, len, "[", 1);
        			len = uocapp_cc_sd(buffer, len, sensors_is_present(coap_options[iCnt].pValue[1] - '0'));
        			len = uocapp_cc_s(buffer, len, "]", 1);
        			ok = TRUE;
        		}
        	}
        }
        if (!ok) {
        	len = 2;
        	memcpy(buffer, "{}", len);
        }

		packetfunctions_reserveHeaderSize(msg, len);
		memcpy(&msg->payload[0], buffer, len);

        // payload marker
        packetfunctions_reserveHeaderSize(msg,1);
        msg->payload[0] = COAP_PAYLOAD_MARKER;

        // CoAP options
        packetfunctions_reserveHeaderSize(msg,2);
        msg->payload[0] = (COAP_OPTION_NUM_CONTENTFORMAT << 4) | 1;
        msg->payload[1] = COAP_MEDTYPE_APPJSON;

        // CoAP header
        coap_header->Code = COAP_CODE_RESP_CONTENT;

        outcome = E_SUCCESS;

        break;

      case COAP_CODE_REQ_POST:
		// Recuperamos el payload
        uocapp_modificar_parametros(msg->payload);

		// La respuesta solo marca que ha sido correcto, no se devuelven datos

        //=== reset packet payload (we will reuse this packetBuffer)
        msg->payload = &(msg->packet[127]);
        msg->length = 0;

        // payload marker
        packetfunctions_reserveHeaderSize(msg,1);
        msg->payload[0] = COAP_PAYLOAD_MARKER;

        // CoAP options
        packetfunctions_reserveHeaderSize(msg,2);
        msg->payload[0] = (COAP_OPTION_NUM_CONTENTFORMAT << 4) | 1;
        msg->payload[1] = COAP_MEDTYPE_APPJSON;

        // CoAP header
        coap_header->Code = COAP_CODE_RESP_CHANGED;

        outcome= E_SUCCESS;

        break;
      default:
         // error
         outcome = E_FAIL;
   }

   return outcome;
}

/**
\brief Funcion que se invoca despues de enviar un mensaje CoAP
\param[in] msg  mensaje CoAP enviado
\param[in] error salida
*/
void uocapp_coap_sendDone(OpenQueueEntry_t* msg, owerror_t error) {
   openqueue_freePacketBuffer(msg);
}

/**
\brief Funcion que se invoca a traves del planificador
\param[in] id id del timer que ha disparado la tarea
*/
void uocapp_tarea_planificada(opentimer_id_t id) {
	scheduler_push_task(uocapp_tarea_planificada_exec,TASKPRIO_COAP);
}

/**
\brief Funcion que se ejecuta desde el planificador como respuesta al timer
*/
void uocapp_tarea_planificada_exec() {
	uint8_t buffer[127];
	uint32_t len = 0;

	len = uocapp_lectura_sensores(buffer, 0);
	uocapp_coap_send(buffer, len);
	len = uocapp_lectura_sensores(buffer, 3);
	uocapp_coap_send(buffer, len);

	len = uocapp_lectura_sensores_estado(buffer);
	uocapp_coap_send(buffer, len);

	buffer[0] = 0;
	len = 0;
	len = uocapp_cc_s(buffer, len, "{\"f\":", 5);
	len = uocapp_cc_ld(buffer, len, uocapp_vars.sincronizacion);
	len = uocapp_cc_s(buffer, len, "}", 1);
	uocapp_coap_send(buffer, len);
}

/**
\brief Funcion que se invoca para leer los sensores de estado
\param[out] buffer  payload del mensaje CoAP a devolver
\return tamanyo del mensaje
*/
uint32_t uocapp_lectura_sensores_estado(uint8_t* buffer) {
	uint8_t iCnt = 0;
	uint32_t len = 0;

	buffer[0] = 0;
	len = uocapp_cc_s(buffer, len, "{\"e\":{", 6);
	for (iCnt = 0; iCnt < MAX_ESTADO; iCnt++) {
		if(iCnt > 0) {
			len = uocapp_cc_s(buffer, len, ",", 1);
		}
		if (uocapp_vars.estado[iCnt].sen != NULL) {
			len = uocapp_cc_pv(buffer, len, uocapp_vars.estado[iCnt].idStatus,
					uocapp_vars.estado[iCnt].sen->callbackRead());
		}else {
			len = uocapp_cc_s(buffer, len, "\"", 1);
			len = uocapp_cc_sd(buffer, len, uocapp_vars.estado[iCnt].idStatus);
			len = uocapp_cc_s(buffer, len, "\":-1", 4);
		}
	}
	len = uocapp_cc_s(buffer, len, "}}", 2);
	return len;
}


/**
\brief Funcion que se invoca para leer los sensores de valor de 3 como maximo
\param[out] buffer payload del mensaje CoAP a devolver
\param[in] buffer indice del sensor que se devuelve en primer lugar
\return tamanyo del mensaje
*/
uint32_t uocapp_lectura_sensores(uint8_t* buffer, uint8_t idx) {
	uint8_t iCnt = 0;
	uint32_t len = 0;
	bool ponComa = FALSE;

	buffer[0] = 0;
	len = uocapp_cc_s(buffer, len, "{\"s\":{", 6);
	for (iCnt = idx; iCnt < MAX_SENSORES && iCnt < idx + 3; iCnt++) {
		if (uocapp_vars.sensores[iCnt].activo) {
			if(ponComa) {
				len = uocapp_cc_s(buffer, len, ",", 1);
			}
			ponComa = TRUE;
			if (uocapp_vars.sensores[iCnt].sen != NULL) {
				len = uocapp_cc_pv(buffer, len, uocapp_vars.sensores[iCnt].id,
						uocapp_vars.sensores[iCnt].sen->callbackRead());
			}else {
				len = uocapp_cc_sd(buffer, len, uocapp_vars.sensores[iCnt].id);
				len = uocapp_cc_s(buffer, len, ":-1", 3);
			}
		}
	}
	len = uocapp_cc_s(buffer, len, "}}", 2);
	return len;
}

/**
\brief Funcion que se invoca para enviar la informacion a la aplicacion web
\param[in] respuesta payload del mensaje CoAP enviado
\param[in] len longitud del payload
*/
void uocapp_coap_send(uint8_t* respuesta, uint32_t len) {
   OpenQueueEntry_t* msg;
   owerror_t outcome;
   int iCnt;

   for (iCnt = 0; iCnt < MAX_SERVIDORES; iCnt++) {
	 if (uocapp_vars.servidores[iCnt].activo) {
		// create a CoAP RD packet
		msg = openqueue_getFreePacketBuffer(COMPONENT_UOCAPP);
		if (msg == NULL) {
			openserial_printError(
				COMPONENT_UOCAPP,
				ERR_NO_FREE_PACKET_BUFFER,
				(errorparameter_t)0,
				(errorparameter_t)0
			);
			openqueue_freePacketBuffer(msg);
			return;
		}

		// ownership
		msg->creator = COMPONENT_UOCAPP;
		msg->owner = COMPONENT_UOCAPP;

		// payload
        packetfunctions_reserveHeaderSize(msg, len);
        memcpy(&msg->payload[0], respuesta, len);

		// payload marker
		packetfunctions_reserveHeaderSize(msg, 1);
		msg->payload[0] = COAP_PAYLOAD_MARKER;

		// CoAP options
		packetfunctions_reserveHeaderSize(msg, 2);
		msg->payload[0] = ((COAP_OPTION_NUM_CONTENTFORMAT - COAP_OPTION_NUM_URIPATH) << 4) | 1;
		msg->payload[1] = COAP_MEDTYPE_APPJSON;

		// location-path0
		packetfunctions_reserveHeaderSize(msg, sizeof(recursoCoapAppWeb));
		memcpy(&msg->payload[1], recursoCoapAppWeb, sizeof(recursoCoapAppWeb)-1);
		msg->payload[0] = ((COAP_OPTION_NUM_URIPATH) << 4) | (sizeof(recursoCoapAppWeb)-1);

        // metadata
		msg->l4_destination_port = uocapp_vars.servidores[iCnt].puerto;
		msg->l3_destinationAdd.type = ADDR_128B;
		memcpy(&msg->l3_destinationAdd.addr_128b[0], uocapp_vars.servidores[iCnt].ipv6Addr, 16);

		// send
		outcome = opencoap_send(
			msg,
			COAP_TYPE_NON,
			COAP_CODE_REQ_POST,
			2, // 2 options, URIPATH y CONTENTFORMAT
			&uocapp_vars.coap // descripcion CoAP del llamante
		);

		// avoid overflowing the queue if fails
		if (outcome == E_FAIL) {
			openqueue_freePacketBuffer(msg);
		}
	  }
	}
}


/**
\brief Funcion que recibe la peticion POST CoAP y modifica los parámetros de actuacion
\param[in] peticion de modificacion de parametros en formato JSON
*/
void uocapp_modificar_parametros(uint8_t* peticion) {
	// La peticion llega en formato JSON
	// {<id_pametro>:<valor parametro>, ...}
	uint8_t idParamero;
	int8_t control;
	uint8_t buffer[100];
	bool esValor;

	if (peticion[0] != 0) {
		control = 0;
		esValor = FALSE;
		while (control != -1) {
			control = uocapp_parsea_peticion(peticion, control, buffer);
			if (esValor) {
				switch(idParamero) {
				case PARAM_SINCRONIZACION:
					uocapp_vars.sincronizacion = uocapp_atoi(buffer, 10);
					break;
				case PARAM_PERIODO:
					uocapp_modifica_planificacion(uocapp_atoi(buffer, 10));
					break;
				case PARAM_SENSORES_ACTIVOS:
					uocapp_parsea_lista_sensores(buffer, uocapp_trata_sensor);
					break;
				case PARAM_SERVIDORES:
					uocapp_parsea_param_servidor(buffer, uocapp_trata_direccion_servidor);
					break;
				}
			}else {
				buffer[uocapp_len(buffer)-1] = 0;
				idParamero = uocapp_atoi(buffer+1, 10);
			}
			esValor = !esValor;
		}
	}
}

/**
\brief Funcion que parsea la peticion JSON
\param[in] cadena en formato JSON
\param[in] init indicador de por donde seguir el parseo
\param[out] buffer valor del parametro o del valor
\return indicador de por donde seguir el parseo
*/
uint32_t uocapp_parsea_peticion(uint8_t* json, uint32_t init, uint8_t* buffer) {
	uint8_t c = 0;
	bool enString = FALSE;
	uint32_t cinicio = 0;
	uint32_t iCnt = 0;
	uint32_t len_json = uocapp_len(json);
	for (iCnt = init; iCnt < len_json; iCnt++) {
		c = json[iCnt];
		if (c == '{' && !enString) {
			cinicio = iCnt + 1;
		}
		else if ((c == ',' || c==':') && !enString) {
			if (cinicio != 0) {
				memcpy(buffer, &json[cinicio], iCnt-cinicio);
				buffer[iCnt-cinicio] = 0;
				return iCnt;
			}
			cinicio = iCnt + 1;
		}
		else if (c == '}' && !enString) {
			if (cinicio != 0) {
				memcpy(buffer, &json[cinicio], iCnt-cinicio);
				buffer[iCnt-cinicio] = 0;
				return -1;
			}
			cinicio = 0;
		}
		else if (c == '"') {
			enString = !enString;
		}
	}
	return -1;
}

/**
\brief Parsea la lista de sensores activos (ids separados por ,)
\param[in] lista de sensores
\param[in] callback funcion de callback para activar el sensor
*/
void uocapp_parsea_lista_sensores(uint8_t* lista, _uocapp_trata_sensor callback) {
	// "1,2,3"
	uint8_t iCnt = 0;
	uint8_t enNumero = 0;
	uint8_t buffer[10];
	uint32_t len_lista = uocapp_len(lista);

	// Activa todos los sensores
	if (len_lista == 3 && lista[1] == 'T') {
		for (iCnt = 0; iCnt < MAX_SENSORES; iCnt++) {
			uocapp_vars.sensores[iCnt].activo = TRUE;
		}
	}
	else if (len_lista == 3 && lista[1] == 'N') {
		// Inactiva todos los sensores
		for (iCnt = 0; iCnt < MAX_SENSORES; iCnt++) {
			uocapp_vars.sensores[iCnt].activo = FALSE;
		}
	}
	else {
		// Inactiva todos los sensores
		for (iCnt = 0; iCnt < MAX_SENSORES; iCnt++) {
			uocapp_vars.sensores[iCnt].activo = FALSE;
		}
		// Activa los sensores que haya en la lista
		for (iCnt = 0; iCnt < len_lista; iCnt++) {
			if (lista[iCnt] < '0' || lista[iCnt] > '9') {
				if (enNumero) {
					memcpy(buffer, &lista[enNumero], iCnt-enNumero);
					buffer[iCnt-enNumero] = 0;
					callback(uocapp_atoi(buffer, 10));
				}
				enNumero  = 0;
			}
			else if (!enNumero){
				enNumero = iCnt;
			}
		}
	}
}

/**
\brief Activa el servidor dado
\param[in] idSensor identificador del sensor
*/
void uocapp_trata_sensor(uint8_t idSensor) {
	uint8_t iCnt = 0;
	for (iCnt = 0; iCnt < MAX_SENSORES; iCnt++) {
	   if (uocapp_vars.sensores[iCnt].id == idSensor) {
		  uocapp_vars.sensores[iCnt].activo = TRUE;
	   }
	}
}

/**
\brief Parsea la lista de servidores (lista de direcciones [ipv6]:puerto)
\param[in] lista lista de servidores
\param[in] callback funcion de callback para tratar un servidor
*/
void uocapp_parsea_param_servidor(uint8_t* servidor, _uocapp_trata_direccion_servidor callback) {
	// "0FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
	uint32_t iCnt = 0;
	uint32_t jCnt = 0;
	uint8_t idServidor = 0;
	uint8_t aux = 0;
	uint8_t strservidor[33];
	uint8_t strpuerto[5];
	uint32_t len = uocapp_len(servidor);

	idServidor = servidor[1] - '0';
	if (servidor[2] == '-') {
		for (iCnt = idServidor; iCnt < MAX_SERVIDORES; iCnt++) {
			uocapp_vars.servidores[iCnt].activo = FALSE;
		}
	}else {
		for (iCnt = 0; iCnt < len - 7; iCnt++) {
			if (servidor[iCnt + 2] == ':') {
				for (jCnt = 0; jCnt < 39 - len; jCnt++) {
					strservidor[aux++] = '0';
				}
			}else
				strservidor[aux++] = servidor[iCnt + 2];
		}
		strservidor[32] = 0;
		memcpy(strpuerto, servidor + (len - 5), 4);
		strpuerto[4] = 0;
		callback(strservidor, (uint16_t)uocapp_atoi(strpuerto, 16), idServidor);
	}
}

/**
\brief Asigna la ip:puerto a cada servidor de la lista de configuracion
\param[in] ipv6 ip version 6
\param[in] puerto puerto del servidor
\param[in] idServidor id del servidor en la lista
*/
void uocapp_trata_direccion_servidor(uint8_t* ipv6, uint16_t puerto, uint8_t idServidor) {
	uint8_t ipv6_bytes[16];
	uint8_t ipv6_g[5];
	uint32_t iCnt = 0;
	uint32_t cinicio = 0;
	uint32_t iCntByte = 0;

	for (iCnt = 0; iCnt < 8; iCnt++) {
		memset(ipv6_g, 0, 5);
		memcpy(ipv6_g, ipv6 + cinicio, 4);
		cinicio += 4;
		int ig = uocapp_atoi(ipv6_g, 16);
		ipv6_bytes[iCntByte++] = ig / 256;
		ipv6_bytes[iCntByte++] = ig % 256;
	}

	uocapp_vars.servidores[idServidor].puerto = puerto;
	uocapp_vars.servidores[idServidor].activo = TRUE;
	memcpy(uocapp_vars.servidores[idServidor].ipv6Addr, ipv6_bytes, 16);
}

/**
\brief Funcion que se invoca para modificar la planificación de la tarea
\param[in] periodo nuevo periodo de la tarea planificada
*/
void uocapp_modifica_planificacion(uint32_t periodo) {
	uocapp_vars.tarea.periodo = periodo * 1000;

	if (periodo == 0 && uocapp_vars.tarea.enEjecucion) {
		opentimers_stop(uocapp_vars.tarea.timerId);
		uocapp_vars.tarea.enEjecucion = FALSE;
	}
	else if (periodo > 0) {
		opentimers_setPeriod(uocapp_vars.tarea.timerId, TIME_MS, uocapp_vars.tarea.periodo);
		if (!uocapp_vars.tarea.enEjecucion) {
			opentimers_restart(uocapp_vars.tarea.timerId);
		}

	}
}

/**
\brief Funcion que calcula la longitud de una cadena
\param[in] s cadena
\return longitud de una cadena
*/
uint32_t uocapp_len(uint8_t* s) {
	uint32_t ret = 0;
	if (s != NULL) {
		for (ret = 0; s[ret] != 0; ret++);
	}
	return ret;
}

/**
\brief Prepara el mensaje JSON de un parametro:valor
\param[in,out] buffer mensaje en la que se almacena el JSON
\param[in] init indice dentro del mensaje donde se escribe
\param[in] v parametro
\param[in] p valor
\return longitud del mensaje actualizado
*/
uint32_t uocapp_cc_pv(uint8_t* buffer, uint32_t init, uint8_t p, uint32_t v) {
	uint32_t len = init;
	len = uocapp_cc_s(buffer, len, "\"", 1);
	len = uocapp_cc_sd(buffer, len, p);
	len = uocapp_cc_s(buffer, len, "\":", 2);
	len = uocapp_cc_ld(buffer, len, v);
	return len;
}

/**
\brief Incluye en el mensaje una cadena (equivalente strcat)
\param[in,out] buffer mensaje en la que se almacena el JSON
\param[in] init indice dentro del mensaje donde se escribe
\param[in] c cadena
\param[in] l longitud de la cadena
\return longitud del mensaje actualizado
*/
uint32_t uocapp_cc_s(uint8_t* buffer, uint32_t init, char* c, uint8_t l) {
	uint32_t len = init;
	uint8_t iCnt = 0;
	for (iCnt = 0; iCnt < l; iCnt++) {
		buffer[len++] = c[iCnt];
	}
	buffer[len] = 0;
	return len;
}

/**
\brief Incluye en el mensaje un entero de 32 bits en formato decimal (equivalente sprintf("%ld"))
\param[in,out] buffer mensaje en la que se almacena el JSON
\param[in] init indice dentro del mensaje donde se escribe
\param[in] ld valor a incluir
\return longitud del mensaje actualizado
*/
uint32_t uocapp_cc_ld(uint8_t* buffer, uint32_t init, uint32_t ld) {
	uint32_t len = init;
	uint32_t cld = ld;
	uint32_t max = 1000000000;
	uint8_t s = 0;
	int ya = 0;
	while (max > 1) {
		s = cld / max;
		if (s > 0 || (s == 0 && ya)) {
			ya = 1;
			buffer[len++] = digitos[s];
		}
		cld = cld % max;
		max /= 10;
	}
	buffer[len++] = digitos[cld];
	buffer[len] = 0;
	return len;
}

/**
\brief Incluye en el mensaje un entero de 16 bits en formato decimal (equivalente sprintf("%d"))
\param[in,out] buffer mensaje en la que se almacena el JSON
\param[in] init indice dentro del mensaje donde se escribe
\param[in] ld valor a incluir
\return longitud del mensaje actualizado
*/
uint32_t uocapp_cc_d(uint8_t* buffer, uint32_t init, uint16_t ld) {
	uint32_t len = init;
	uint16_t val = ld;
	if (val / 10000 != 0) {
		buffer[len++] = digitos[val / 10000];
	}
	val = val % 10000;
	if (val / 1000 != 0 || len != init) {
		buffer[len++] = digitos[val / 1000];
	}
	val = val % 1000;
	if (val / 100 != 0 || len != init) {
		buffer[len++] = digitos[val / 100];
	}
	val = val % 100;
	if (val / 10 != 0 || len != init) {
		buffer[len++] = digitos[val / 10];
	}
	val = val % 10;
	buffer[len++] = digitos[val];
	buffer[len] = 0;
	return len;
}

/**
\brief Incluye en el mensaje un entero de 8 bits en formato decimal (equivalente sprintf("%sd"))
\param[in,out] buffer mensaje en la que se almacena el JSON
\param[in] init indice dentro del mensaje donde se escribe
\param[in] ld valor a incluir
\return longitud del mensaje actualizado
*/
uint32_t uocapp_cc_sd(uint8_t* buffer, uint32_t init, uint8_t ld) {
	uint32_t len = init;
	uint8_t val = ld;
	if (val / 100 != 0) {
		buffer[len++] = digitos[val / 100];
	}
	val = val % 100;
	if (val / 10 != 0 || len != init) {
		buffer[len++] = digitos[val / 10];
	}
	val = val % 10;
	buffer[len++] = digitos[val];
	buffer[len] = 0;
	return len;
}

/**
\brief Incluye en el mensaje un entero de 8 bits en formato hexadecimal (equivalente sprintf("%X"))
\param[in,out] buffer mensaje en la que se almacena el JSON
\param[in] init indice dentro del mensaje donde se escribe
\param[in] ld valor a incluir
\return longitud del mensaje actualizado
*/
uint32_t uocapp_cc_hex(uint8_t* buffer, uint32_t init, uint8_t ld) {
	uint32_t len = init;
	buffer[len++] = digitos_hex[ld / 16];
	buffer[len++] = digitos_hex[ld % 16];
	buffer[len] = 0;
	return len;
}

/**
\brief Transforma una cadena de caracteres en un numero dada la base (equivalente strtol o atoi)
\param[in] s cadena de caracteres
\param[in] base base de la conversion (10,16)
\return numero resultado de la conversion
*/
uint32_t uocapp_atoi(uint8_t* s, uint8_t base) {
	uint32_t ret = 0;
	uint32_t mult = 1;
	uint32_t iCnt = 0;
	uint8_t calc = 0;
	uint32_t len = uocapp_len(s);
	for (iCnt = 0; iCnt < len; iCnt++){
		calc = 0;
		if (s[len - 1 - iCnt] >= '0' && s[len - 1 - iCnt] <= '9') {
			calc = s[len - 1 - iCnt] - '0';
		}else if (s[len - 1 - iCnt] >= 'a' && s[len - 1 - iCnt] <= 'f') {
			calc = s[len - 1 - iCnt] - 'a' + 10;
		}else if (s[len - 1 - iCnt] >= 'A' && s[len - 1 - iCnt] <= 'F') {
			calc = s[len - 1 - iCnt] - 'A' + 10;
		}
		ret = ret + mult * calc;
		mult *= base;
	}
	return ret;
}

